<?
/*
 * $Horde$
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

class Type {
    function isValid(&$vars, $value, &$message) {
	$message = "<b>Error:</b> Type::isValid called - should be overridden<br />";
	return false;
    }

    function getDisplayHtml(&$vars, $name, $value) {
	return empty($value) ? '&nbsp;' : htmlspecialchars($value);
    }

    function getFormHtml(&$vars, $name, $value) {
	return "<b>Error:</b> Type::getFormHtml called - should be overridden<br />";
    }
}

class IntType extends Type {
    function isValid(&$vars, $value, &$message) {
	if (empty($value) || preg_match('/^[0-9]+$/', $value))
	    return true;

	$message = 'must only contain numbers';
	return false;
    }

    function getFormHtml(&$vars, $name, $value) {
	return "<input type=\"text\" name=\"$name\" value=\"$value\" />";
    }
}

class TextType extends Type {
    function isValid(&$vars, $value, &$message) {
	return true;
    }

    function getFormHtml(&$vars, $name, $value) {
	return "<input type=\"text\" name=\"$name\" value=\"$value\" size=\"35\" />";
    }
}

class LongTextType extends Type {
    function isValid(&$vars, $value, &$message) {
	return true;
    }

    function getFormHtml(&$vars, $name, $value) {
	return '<textarea name="' . $name . '" cols="80" rows="6" wrap="hard">' . $value . "</textarea>";
    }
}

class BooleanType extends Type {
    function isValid(&$vars, $value, &$message) {
	return true;
    }

    function getFormHtml(&$vars, $name, $value) {
	return '<input type="checkbox" name="' . $name . '"' . ($value ? ' checked="yes"' : '') . '>';
    }
}

class EmailType extends Type {
    function isValid(&$vars, $value, &$message) {
	// FIXME:
	return true;
    }

    function getDisplayHtml(&$vars, $name, $value) {
	if (strpos($value, '@') === false)
	    return $value;

	// see if we've got a name for the address
	$name = $vars->getVar($name . '_name');

	$value = urlencode($value);
	$name = htmlspecialchars($name);

	if (!empty($name)) {
	    return "<a href=\"mailto:$value\">$name</a>";
	}

	return "<a href=\"mailto:$value\">$value</a>";
    }

    function getFormHtml(&$vars, $name, $value) {
	return "<input type=\"text\" name=\"$name\" value=\"$value\" size=\"35\" />";
    }    
}

class EnumType extends Type {
    function EnumType(&$values) {
	$this->values = $values;
    }

    function isValid(&$vars, $value, &$message) {
	if (isset($this->values[$value]))
	    return true;

	$message = 'hey, stop screwing with my post vars';
	return false;
    }

    function getDisplayHtml(&$vars, $name, $value) {
	$val = $this->values[$value];
	return empty($val) ? '&nbsp;' : $val;
    }

    function getFormHtml(&$vars, $name, $value) {
	foreach ($this->values as $optvalue => $display) {
	    $selected = ($optvalue == $value ? ' selected="yes"' : '');
	    $options .= " <option value=\"$optvalue\"$selected>$display</option>\n";
	}

	return "<select name=\"$name\">\n" . $options . "</select>";
    }
}

class UserType extends Type {
    function UserType(&$backend) {
	$this->backend = $backend;
    }

    function isValid(&$vars, $value, &$message) {
	$uid = $this->backend->getUserFromEmail($value);

	if (!$uid) {
	    $message = "no such user " . htmlspecialchars($value);
	    return false;
	}

	return true;
    }

    function getFormHtml(&$vars, $name, $value) {
	return "<input type=\"text\" name=\"$name\" value=\"$value\" size=\"35\"/>";
    }
}

class Variable {
    function Variable($humanName, $varName, &$type, $required, $readonly = false, $hidden = false) {
	$this->humanName = $humanName;
	$this->varName   = $varName;
	$this->type      = $type;
	$this->required  = $required;
	$this->readonly  = $readonly;
	$this->hidden    = $hidden;
    }

    function setDefault($value) {$this->defValue = $value;}

    function  getHumanName() {return $this->humanName;}
    function  getVarName() {return $this->varName;}
    function &getType() {return $this->type;}
    function  getDefault() {return $this->defValue;}
    function  isRequired() {return $this->required;}
    function  isReadonly() {return $this->readonly;}
    function  isHidden() {return $this->hidden;}

    function getDisplayHtml(&$vars, $value) {
	return $this->type->getDisplayHtml($vars, $this->varName, $value);
    }

    function getFormHtml(&$vars, $value) {
	return $this->type->getFormHtml($vars, $vars->getPrefix() . $this->varName, $value);
    }

    function isValid(&$vars, $value, &$message) {
	return $this->type->isValid($vars, $value, $message);
    }
}

class FormRendererBase {
    function outputInput($humanName, $input, $error) {
	echo "<b>Error:</b> FormRenderer::outputInput called, should be overridden\n";
    }

    function outputDisplay($humanName, $display, $error) {
	echo "<b>Error:</b> FormRenderer::outputDisplay called, should be overridden\n";
    }

    function outputSubmit($submit) {
	echo "<b>Error:</b> FormRenderer::outputSubmit called, should be overridden\n";
    }
}

class NullRenderer extends FormRendererBase {
    function outputInput($humanName, $input, $error) {}
    function outputDisplay($humanName, $display, $error) {}
    function outputSubmit($submit) {}
}

class Form {
    function Form($name = '') {
	$this->variables = array();
	$this->name      = $name;
	$this->valid     = 0;
    }

    function addVariable(&$var) {
	array_push($this->variables, $var);
    }

    function open(&$vars, $action, $method = "get") {
	echo "<form action=\"$action\" method=\"$method\">\n";

	if (!empty($this->name))
	    echo '<input type="hidden" name="formname" value="' . $this->name . "\" />\n";

	$vars->pushPrefix('debug');
	$vars->preserveByPost();
	$vars->popPrefix();
    }

    function render(&$renderer, &$vars, $readonly = false, $preserve = true) {
	foreach ($this->variables as $key => $var) {
	    $varname   = $var->getVarName();
	    $humanName = $var->getHumanName();

	    $value = $vars->getVarWasset($varname, $wasset);
	    $error = $this->errors[$varname];

	    if (!$wasset)
		$value = $var->getDefault();

	    $display = $var->getDisplayHtml($vars, $value);
	    $input   = $var->getFormHtml($vars, $value);

	    if ($var->isHidden() && $preserve) {
		$vars->preserveVarByPost($varname);
	    } else {
		if ($readonly || $var->isReadonly()) {
		    if ($preserve)
			$vars->preserveVarByPost($varname);

		    $renderer->outputDisplay($humanName, $display, $error);
		} else {
		    if ($var->isRequired())
			$humanName = "* $humanName";

		    $renderer->outputInput($humanName, $input, $error);
		}
	    }
	}
    }

    function preserve($vars) {
	foreach ($this->variables as $key => $var) {
	    $varname = $var->getVarName();
	    $vars->preserveVarByPost($varname);
	}
    }

    function submit(&$renderer, $value = '', $name = '') {
	if (empty($value))
	    $submit = '<input type="submit" />' . "\n";
	else if (empty($name))
	    $submit = '<input type="submit"' . " value=\"$value\" />\n";
	else
	    $submit = '<input type="submit"' . " name=\"$name\" value=\"$value\" />\n";

	$renderer->outputSubmit($submit);
    }

    function close() {
	echo "</form>\n";
    }

    function unsetVars(&$vars) {
	foreach ($this->variables as $key => $var) {
	    $varname = $var->getVarName();
	    $vars->unset($varname);
	}
    }

    function validate(&$vars) {
	foreach ($this->variables as $key => $var) {
	    $varname = $var->getVarName();
	    $value   = $vars->getVar($varname);

	    if (empty($value)) {
		if ($var->isRequired())
		    $this->errors[$varname] = $var->getHumanName() . " is required";
	    } else {
		if (!$var->isValid($vars, $value, $message))
		    $this->errors[$varname] = $message;
	    }
	}

	$this->valid = (sizeof($this->errors) == 0);
    }

    function isValid() {
	return $this->valid;
    }

    function getInfo(&$vars, &$info) {
	$this->_getInfo($vars, $info);
    }

    function _getInfo(&$vars, &$info) {
	foreach ($this->variables as $key => $var) {
	    $varname = $var->getVarName();
	    $value = $vars->getVarWasset($varname, $wasset);

	    if ($wasset)
		$info[$varname] = $value;
	    else if (isset($var->defValue))
		$info[$varname] = $var->defValue;
	}
    }
}

function &getIntType() {
    static $var = "";

    if (empty($var))
	$var = new IntType();

    return $var;
}

function &getTextType() {
    static $var = "";

    if (empty($var))
	$var = new TextType();

    return $var;
}

function &getLongTextType() {
    static $var = "";

    if (empty($var))
	$var = new LongTextType();

    return $var;
}

function &getBooleanType() {
    static $var = "";

    if (empty($var))
	$var = new BooleanType();

    return $var;
}

function &getEmailType() {
    static $var = "";

    if (empty($var))
	$var = new EmailType();

    return $var;
}

function &getUserType(&$backend) {
    static $var = '';
    static $be = '';

    if (empty($var) || $be != $backend) {
	$be = $backend;
	$var = new UserType($backend);
    }

    return $var;
}

?>
